<?php

namespace InterludeSante\Statistics;

use InterludeSante\Statistics\Chart;
use InterludeSante\Statistics\Table;

use InterludeSante\Log;

class Tabs
{
    /**
     * Plugin path.
     *
     * @since  1.0
     * @access private
     */
    private $path;

    /**
     * The current list of items.
     *
     * @since 3.1.0
     * @var array
     */
    public $items;

    public function __construct(string $path)
    {
        $this->path = $path;
        $this->items = [
            [
                'title' => __('Consultation des pages', 'ullis-statistics'),
                'description' => __('Les pages les plus consultées', 'ullis-statistics'),
                'type' => 'table',
                'name' => 'page'
            ],
            [
                'title' => __('Nombre de connexion unique', 'ullis-statistics'),
                'description' => __('Nombre de connexion unique', 'ullis-statistics'),
                'type' => 'chart',
                'name' => 'login',
                'week' => true,
                'month' => true
            ],
            [
                'title' => __('Sondages', 'ullis-statistics'),
                'description' => __('Statistiques des sondages', 'ullis-statistics'),
                'type' => 'chart',
                'name' => 'survey'
            ],
            [
                'title' => __('Statistiques d\'appels', 'ullis-statistics'),
                'description' => __('Liste les appels passés depuis l\'application', 'ullis-statistics'),
                'type' => 'chart',
                'name' => 'phone',
                'week' => true,
                'month' => true
            ]
        ];
    }

    /**
     * Prepares the Tabs for displaying.
     * @access public
     *
     * @return void
     */
    public function prepare()
    {
        $this->getTabs();
    }

    /**
     * Display block chart.
     * @access public
     *
     * @return void
     */
    public function displayChart($item)
    {
        require $this->path . 'views/admin/panel/chart.php';
    }

    /**
     * Display block table.
     * @access public
     *
     * @return void
     */
    public function displayTable($item)
    {
        require $this->path . 'views/admin/panel/table.php';
    }

    /**
     * Add specific chart data to array
     * @access public
     *
     * @return void
     */
    private function getTabs()
    {
        foreach ($this->items as $key => $item) {
            if ($item["type"] === "chart") {
                if ($item['name'] === "survey") {
                    $this->items[$key]['chart'] = Chart::get($item['name'], 0);
                } else {
                    $this->items[$key]['chart']['week'] = Chart::get($item['name'], null, 'week');
                    $this->items[$key]['chart']['month'] = Chart::get($item['name'], null, 'month');
                }
            }


            if ($item["type"] === "table") {
                $this->items[$key]['table'] = Table::get($item['name']);
            }
        }
    }
}
